;
;
;                           FILE NAME 4Bit5000.ASM
;
;               Project: DALLAS DS5000T MICRO Clock/Callendar
;                          Programmable Event Timer
;                            11.0592 MHz Crystal
;                              LCD = { 20 X 4 }
;
;                               4 bit LCD Code
;                           With user input buttons.
;
;
; ========================================================================
; This project incorperates the Dallas DS5000T in a simple control system.
; In normal mode it simply displays the date and time, and allows setting
; of these functions with user input buttons. The buttons change function
; somewhat in the subroutines. In the normal mode Button #1 moves the
; the user to the Time setting routine. Button #2 moves the user to the
; Alarm and Control setting routine. Below is a diagram of button
; functions for each subroutine.
;
;            Time Set Routine               Setting Alarms
; Button #1       Go Back                      Go Back
; Button #2      Increase                    Daily Alarms
; Button #3      Decrease                Control Alarm Status
; Button #4       Enter                   Control Alarm Set
;
;
;             Daily Alarms     Control alarm status      Control Alarms
; Button #1      Go Back             Go Back                 Go Back
; Button #2     Increase            Go Forward              Increase
; Button #3     Decrease             Go Back                Decrease
; Button #4      Enter             Show Settings              Enter
;                             [ Any button breaks out]
;                             [   of Show Settings   ]
;
;  In all setting routines the arrow indicates which number will be
;  acted on by the buttons.


;                      EQUATES TABLE FOR ASSEMBLER
;                      ===========================


               lcd_data   equ 090h  ; port #1
               lcd_db4    equ 094h  ; port #1.4
               lcd_db5    equ 095h  ; port #1.5
               lcd_db6    equ 096h  ; port #1.6
               lcd_db7    equ 097h  ; port #1.7

               lcd_rs     equ 0b3h  ; port #3.3
               lcd_rw     equ 0b4h  ; port #3.4
               lcd_e      equ 0b5h  ; port #3.5


               Button1    equ 090h  ; port #1.0
               Button2    equ 091h  ; port #1.1
               Button3    equ 092h  ; port #1.2
               Button4    equ 093h  ; port #1.3

               Alarm_out  equ 0b7h  ; port #3.7
               Control    equ 0a0h  ; port #2

               config     equ 28h
               entrymode  equ 6
               offcur     equ 0ch
               clrdsp     equ 01h
               offdsp     equ 0ah
               ondsp      equ 0eh

               pcon       equ 087h
               mcon       equ 0c6h
               ta         equ 0c7h
               acc        equ 0e0h
               sp         equ 081h
               b          equ 0f0h
               dph        equ 083h
               dpl        equ 082h
               psw        equ 0d0h
;               ie         equ 0a8h
;               tmod       equ 089h
;               th1        equ 08dh
;               tl1        equ 08bh
;               scon       equ 098h
;               tcon       equ 088h
;               sbuf       equ 99h
;               ren        equ 9ch
;               ri         equ 98h
;               ti         equ 99h
  





ORG  0
                    ; Reset Vector Established Here.

start1:
               mov   ta,#0aah     ; Timed Access.
               mov   ta,#55h      ;
               mov   pcon,#0      ; Reset WatchDog Timer.
               mov   mcon,#0f8h   ; Turn Off CE2 For Memory Access.
               mov   sp,#2fh      ; Position Stack Above Registers.
               lcall close        ; Make Sure Device Is Closed.

               mov   0a0h,0fh     ; sets output from Imem

; ========================================================================
;        Set up display for display of Day, Date, and Time data.
; First move control data from Xmem to Imem and then print to the LCD
; the stuff that doesn't need to be reprinted each time while the time
; and date are displayed
; ========================================================================
msg:
               lcall bounce
               mov   dpl,#00h     ; Set data pointer and retreave
               mov   dph,#1ch     ; alarm status data
               movx  a,@dptr
               mov   20h,a
               
               lcall resetlcd4  ; Place Initial Data For LCD
               mov   a,#1       ; Date:  Time: Etc...
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '*-                -*',0
               mov   a,#2
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '  ******    ******  ',0
               mov   a,#3
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    '{ DATE }',0
               mov   a,#3
               mov   b,#11
               lcall placecur
               lcall prtlcd4
               db    '{ TIME }',0
               mov   a,#4           ; Place Symbols To Seperate
               mov   b,#1           ; Date & Time Data.
               lcall placecur
               lcall prtlcd4
               db    '  /  /  ',0
               mov   a,#4
               mov   b,#11
               lcall placecur
               lcall prtlcd4
               db    '  :  :  ',0
               mov   r0,#offcur
               lcall wrlcdcom4

; ========================================================================
;              Time display and alarm and control checking
;      Main loop of program. During normal operation the program stays
;      within this loop. Going to the subroutines to display the data.
;      and checking for button presses. Also checking for alarms and
;      control outputs, and their off times.
; ========================================================================
continue:
               jnb   Button1,set_time1    ; Button1 goes to set time
               jnb   Button2,set_alarms1  ; Button2 goes to set alarms

               lcall open              ; Open chip to read data
               mov   r0,#8             ; How many bytes to read
f:             lcall rbyte             ; read a byte
               mov   @r0,a             ; put it in a register
               djnz  r0,f              ; do it until you got all
               lcall day               ; display the day of the week
               lcall date_time         ; display the date and time data
               sjmp  check_alarm


set_time1:                             ; Turns short jump into long one.
               ljmp  set_time          ; to go to time setting routine.

set_alarms1:                           ; Turns short jump into long one.
               ljmp  set_alarms        ; to go to alarm or control
                                       ; setting routines.

; ========================================================================
;                   Routine to check daily alarms
; This routine takes the day number (r4) and mutilplies it by 10 to get
; the LSB of the external memory address which holds the alarm time for
; that day. If the memory location is empty it won't check for a alarm.
; If there is data the routine compares hour data from clock (5=R5)and,
; if its the same as the alarm hour,(first memory location ie: 1E00h for
; day one) it then checks to see if the minutes are the same. (R6)
; (second address) If it all matches the alarm goes off for one minute.
; (minutes no longer match after one minute)
; ========================================================================
check_alarm:
               mov   a,r4                 ; Day number
               dec   a                    ; decrease it by one
               mov   b,#10h               ;
               mul   ab                   ; multiply by 10
               mov   dpl,a                ; set LSB of DPTR
               mov   dph,#1eh             ; set MSB of DPTR

               movx  a,@dptr              ; get what is at the address
               cjne  a,#00,theres_one_set ; Nothing there, forget it
               sjmp  check_control        ; go check control
theres_one_set:
               cjne  a,5,forget_it        ; compare data with hour
               inc   dpl                  ; go one more address
               movx  a,@dptr              ; get it
               cjne  a,6,forget_it        ; compare it with minutes
               clr   alarm_out            ; they both match turn on alarm
               sjmp  check_control
forget_it:
               setb  alarm_out            ; They don't match turn off
                                          ; alarm bit.

; ========================================================================
;               Routine to check ONtimes for controls
; This routine uses the bit addressable Internal Ram location 20h to see
; if there are control outputs set. Each bit corresponds to a relay
; ( 00 = relay #1 ) This data is stored in Xmem when ever a control is set
; and then copied to Imem. ( begining of program )
; If no bits are set then we skip this routine. If a bit is set then
; it determines which one and then sets R0 with LSB of Xmem address, and
; 10h with output data for output port ( Port 2 ). Then it calls the
; routine to check on the date and time.
; ========================================================================
check_control:
               mov   a,20h                ; get status bits
               cjne  a,#00h,check_one     ; none set = no controls set
               sjmp  continue             ; go on displaying time

check_one:
               jnb   00h,check_two        ; no bit go to next one
               mov   r0,#11h              ; bit set put this data in
               mov   10h,#01h             ; registers for subroutine
               lcall check_if_alarm       ; go to the subroutine
check_two:
               jnb   01h,check_three
               mov   r0,#21h
               mov   10h,#02h
               lcall check_if_alarm
check_three:
               jnb   02h,check_four
               mov   r0,#31h
               mov   10h,#04h
               lcall check_if_alarm
check_four:
               jnb   03h,check_five
               mov   r0,#41h
               mov   10h,#08h
               lcall check_if_alarm
check_five:
               jnb   04h,check_six
               mov   r0,#51h
               mov   10h,#10h
               lcall check_if_alarm
check_six:
               jnb   05h,check_seven
               mov   r0,#61h
               mov   10h,#20h
               lcall check_if_alarm
check_seven:
               jnb   06h,check_eight
               mov   r0,#71h
               mov   10h,#40h
               lcall check_if_alarm
check_eight:
               jnb   07h,control_on
               mov   r0,#81h
               mov   10h,#80h
               lcall check_if_alarm

; ========================================================================
;                Routine to check OFFtimes for controls
; This routine checks to see if a bit is set on the output port and then
; uses this to determine where to look to see if it is time to turn the
; control off. Each bit on the port corresponds to a relay ( A0h = #1 )
; Much like the routine above if it finds a bit set it then
; sets the address ( R0 ) and the output data ( 10h ) and then calls the
; subroutine that checks the turn off date and times.
; ========================================================================
control_on:
               mov   a,0a0h               ; check output status
               cjne  a,#00h,which_one     ; none on so go back
               ljmp  continue             ; and display time

which_one:
               jnb   0a0h,two             ; if not set check next one
               mov   r0,#16h              ; bit set put this data in
               mov   10h,#0feh            ; registers for subroutine
               lcall check_off            ; go to the subroutine
two:
               jnb   0a1h,three
               mov   r0,#26h
               mov   10h,#0fdh
               lcall check_off
three:
               jnb   0a2h,four
               mov   r0,#36h
               mov   10h,#0fbh
               lcall check_off
four:
               jnb   0a3h,five
               mov   r0,#46h
               mov   10h,#0f7h
               lcall check_off
five:
               jnb   0a4h,six
               mov   r0,#56h
               mov   10h,#0efh
               lcall check_off
six:
               jnb   0a5h,seven
               mov   r0,#66h
               mov   10h,#0dfh
               lcall check_off
seven:
               jnb   0a6h,eight
               mov   r0,#76h
               mov   10h,#0bfh
               lcall check_off
eight:
               jnb   0a7h,end_check
               mov   r0,#86h
               mov   10h,#7fh
               lcall check_off
end_check:
               ljmp   continue

; ========================================================================
;                SubRoutine to check ON times for controls
; This routine does the real work of checking for controls. It takes the
; address data ( R0 ) and then gets the date data ( ie 1C02 for relay 1 )
; it compares it with todays date. If there is a match the address is
; decreased by one ( month data ) and this is checked for a match. Then
; two is added to the address ( year data ) its checked, then the address
; is increased  ( hour data ) its checked, address is increased again
; ( minutes data ) its checked. If all data matches then the control is
; set by ORing the output data ( 10h ) with the existing data in memory
; location 21h. This way only the data for the relay that is turned on is
; changed. The data in 21h is then sent to the output port. I used
; a Imem location to store the output data instead of just using the
; output port so I wouldn't lose the data on a power loss. Since the chip
; is battery backed Imem is safe.
;       If at any point along the way the data does not match, the
; routine drops out to go back to the place it was called from.
;
;       The way I configured the memory locations are confusing
;       and I should have done it in a more consecutive order,
;       but it all came about because of the order the time data
;       is stored in the DS5000T.
; ========================================================================
check_if_alarm:
               mov   dpl,r0            ; set at on_date in xmem
               mov   dph,#1ch
               movx  a,@dptr
               cjne  a,3h,no_alarm     ; compare it with the date
               dec   dpl               ; set at on_month in xmem
               movx  a,@dptr
               cjne  a,2h,no_alarm     ; compare it with the month
               mov   a,dpl
               add   a,#2              ; set at on_year in xmem
               mov   dpl,a
               movx  a,@dptr
               cjne  a,1h,no_alarm     ; compare it with the year
               inc   dpl               ; set at on_hour in xmem
               movx  a,@dptr
               cjne  a,5h,no_alarm     ; compare it with the hour
               inc   dpl               ; set at on_min in xmem
               movx  a,@dptr
               cjne  a,6h,no_alarm     ; compare it with the minutes
               mov   a,0fh             ; get present output data
               orl   a,10h             ; or it with new data
               mov   0fh,a             ; save it in Imem
               mov   dpl,#01h          ; set LSB of DPTR
               mov   dph,#1ch          ; set MSB of DPTR
               movx  @dptr,a           ; send status byte to Xmem
               mov   0a0h,0fh          ; send it to the port


no_alarm:
               ret
; ========================================================================
;                SubRoutine to check Off times for controls
; This routine does the same thing as the one above except it works with
; the off times. If everything matches then the output data ( 10h ) is
; ANDed with the data in 21h. This way it only changes the relay that is
; supposed to be shut off.
; ========================================================================
check_off:
               mov   dpl,r0            ; set at on_date in xmem
               mov   dph,#1ch
               movx  a,@dptr
               cjne  a,3h,alarm_on
               dec   dpl               ; set at on_month in xmem
               movx  a,@dptr
               cjne  a,2h,alarm_on
               mov   a,dpl
               add   a,#2              ; set at on_year in xmem
               mov   dpl,a
               movx  a,@dptr
               cjne  a,1h,alarm_on
               inc   dpl               ; set at on_hour in xmem
               movx  a,@dptr
               cjne  a,5h,alarm_on
               inc   dpl               ; set at on_min in xmem
               movx  a,@dptr
               cjne  a,6h,alarm_on
               mov   a,0fh             ; get output data
               anl   a,10h             ; and it with new data
               mov   0fh,a             ; save it in Imem
               mov   dpl,#01h          ; set LSB of DPTR
               mov   dph,#1ch          ; set MSB of DPTR
               movx  @dptr,a           ; send status byte to Xmem
               mov   0a0h,0fh          ; send it to the port

alarm_on:
               ret


; ========================================================================
;                           Day lookup table
; Lookup table to determin the day of the week, and display it. The data
; is in R4. This table is used in the normal display and also to display
; the day for alarm setting.
; ========================================================================
day:

               mov   a,#1
               mov   b,#4
               lcall placecur
alarm_day:     mov   a,r4             ; when setting alarms it skips the
               cjne  a,#1,mon         ; first part so the day can be
               lcall prtlcd4          ; displayed at a different location.
               db    '   SUNDAY   ',0
               ret
mon:
               cjne  a,#2,tue
               lcall prtlcd4
               db    '   MONDAY   ',0
               ret
tue:
               cjne  a,#3,wed
               lcall prtlcd4
               db    '  TUESDAY   ',0
               ret
wed:
               cjne  a,#4,thu
               lcall prtlcd4
               db    ' WEDNESDAY  ',0
               ret
thu:
               cjne  a,#5,fri
               lcall prtlcd4
               db    '  THURSDAY  ',0
               ret
fri:
               cjne  a,#6,sat
               lcall prtlcd4
               db    '   FRIDAY   ',0
               ret
sat:
               cjne  a,#7,none
               lcall prtlcd4
               db    '  SATURDAY  ',0

none:
               ret

; =======================================================================
;                      Routine To Show Date & Time.
; This routine gets the time a date data from the clock registers and
; then sends it to the display.
; =======================================================================
date_time:
               mov   a,#4              ; line #4 for date
               mov   b,#1              ; start @ column #1
               lcall placecur          ; place for 1st digit of month
               mov   a,r2              ; load month data
               lcall write_bcd         ; Send it to display.
               mov   a,#4
               mov   b,#4
               lcall placecur          ; place for 1st digit of date
               mov   a,r3              ; date data
               lcall write_bcd
               mov   a,#4
               mov   b,#7              ; place for 1st digit of year
               lcall placecur
               mov   a,r1              ; year data
               lcall write_bcd
               mov   a,#4              ; line #3 for time
               mov   b,#11             ; print hours here
               lcall placecur
               mov   a,r5              ; hour data
               lcall write_bcd
               mov   a,#4
               mov   b,#14
               lcall placecur
               mov   a,r6              ; minutes data
               lcall write_bcd
               mov   a,#4
               mov   b,#17
               lcall placecur
               mov   a,r7              ; seconds data
               lcall write_bcd
               mov   r0,#offcur
               lcall wrlcdcom4         ; turn off cursor
               ret

; ========================================================================
;            Routine to set the clock registers of the 5000T
;   First just to let you know the routine your in we print a message.
; Then we set up the screen like the normal clock mode. Except everything
; is static and there is a arrow next to the day. The buttons change it
; either up or down and when button 4 is pressed the arrow moves to the
; next number and it can be changed. You can simply go to the on thing
; you want to change and thats all you need to change.
;       Remember what you see is what you get. If you mess up you have
; reset by pressing button 1 and not saving data, and then starting all
; over again.
; ========================================================================
set_time:
               lcall bounce
               lcall resetlcd4                ; First set up display
               mov   a,#1                     ; to let you know your
               mov   b,#0                     ; setting the time.
               lcall placecur
               lcall prtlcd4
               db    '********************',0
               mov   a,#3
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '*                  *',0
               mov   a,#2
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '*  TIME SET MODE   *',0
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '********************',0
               mov   r0,#offcur
               lcall wrlcdcom4
               lcall big_delay2
               
msg1:
               lcall resetlcd4                ; Place Data For LCD
               mov   a,#1                     ; so you will know what
               mov   b,#0                     ; your setting.
               lcall placecur
               lcall prtlcd4
               db    '*-                -*',0
               mov   a,#2
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '  ******    ******  ',0
               mov   a,#3
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    '{ DATE }',0
               mov   a,#3
               mov   b,#11
               lcall placecur
               lcall prtlcd4
               db    '{ TIME }',0
               mov   a,#4
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    '  /  /  ',0
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#11
               lcall placecur
               lcall prtlcd4
               db    '  :  :  ',0
               mov   r0,#offcur
               lcall wrlcdcom4

               lcall open             ; get the time data and display it
               mov   r0,#8
e:             lcall rbyte
               mov   @r0,a
               djnz  r0,e
               lcall day
               lcall date_time


               mov   a,#1
               mov   b,#3
               lcall placecur
               lcall prtlcd4          ; position cursor and show arrow
               db    126,0            ; so you can tell what your setting
               mov   r0,#offcur
               lcall wrlcdcom4


wait:
               jnb   Button2,up        ; wait here until a button
               jnb   Button3,down      ; is pressed
               jnb   Button4,enter
               jnb   Button1,go_back
               sjmp  wait

enter:
               lcall bounce            ; all done go to next data
               sjmp  set_month

up:
               lcall bounce            ; increases number by one
               inc   r4
               cjne  r4,#8h,oka
               mov   r4,#1h
               sjmp  oka

oka:
               lcall day               ; the data is good display it
               sjmp  wait

down:
               lcall bounce            ; decreases number by one
               dec   r4
               cjne  r4,#0h,oka
               mov   r4,#7h
               lcall day
               sjmp  wait


go_back:
               lcall bounce
               ljmp  msg               ; button1 will take you back
                                       ; to where you came from.

set_month:
               lcall bounce
               mov   a,#1
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    ' ',0             ; erase arrow next to day
               mov   a,#4
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    '/',0             ; makes sure slash is in place
               mov   a,#4              ; if you come back here from
               mov   b,#0              ; set date
               lcall placecur
               lcall prtlcd4
               db    126,0             ; put arrow next to month
               mov   r0,#offcur
               lcall wrlcdcom4
                        
wait1:
               jnb   Button2,up1       ; wait here
               jnb   Button3,down1
               jnb   Button4,enter1
               jnb   Button1,go_back1
               sjmp  wait1

enter1:
               lcall bounce            ; all done go to set date
               sjmp  set_date

up1:
               lcall bounce            ; increase number by one
               mov   a,r2
               lcall check_bcd         ; make sure its legal BCD
               mov   r2,a
               cjne  r2,#13h,ok1
               mov   r2,#1h
               sjmp  ok1

ok1:
               lcall date_time         ; its good go wait some more
               sjmp  wait1

down1:
               lcall bounce            ; decrease number by one
               mov   a,r2
               lcall check_bcd_down    ; as long as it is a legal BCD
               mov   r2,a
               dec   r2
               cjne  r2,#0h,ok1
               mov   r2,#12h
               sjmp  ok1

go_back1:
               lcall bounce
               ljmp  msg1              ; breaks out of routine


set_date:                              ; Same as month but for the date
               lcall bounce
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#6
               lcall placecur
               lcall prtlcd4
               db    '/',0
               mov   a,#4
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   r0,#offcur
               lcall wrlcdcom4

wait2:
               jnb   Button2,up2
               jnb   Button3,down2
               jnb   Button4,enter2
               jnb   Button1,go_back2
               sjmp  wait2

enter2:
               lcall bounce
               sjmp  set_year

up2:
               lcall bounce
               mov   a,r3
               lcall check_bcd
               mov   r3,a
               cjne  r3,#32h,ok2
               mov   r3,#1h
               sjmp  ok2

ok2:
               lcall date_time
               sjmp  wait2

down2:
               lcall bounce
               mov   a,r3
               lcall check_bcd_down
               mov   r3,a
               dec   r3
               cjne  r3,#0h,ok2
               mov   r3,#31h
               sjmp  ok2


go_back2:
               lcall bounce
               ljmp  set_month


set_year:                              ; Same as month but for the year
               lcall bounce
               mov   a,#4
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    '/',0
               mov   a,#4
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#6
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   r0,#offcur
               lcall wrlcdcom4


wait3:
               jnb   Button2,up3
               jnb   Button3,down3
               jnb   Button4,enter3
               jnb   Button1,go_back3
               sjmp  wait3

enter3:
               lcall bounce
               sjmp  set_hours

up3:
               lcall bounce
               mov   a,r1
               lcall check_bcd
               mov   r1,a
               cjne  r1,#9Ah,ok3
               mov   r1,#95h
               ljmp  ok3

ok3:
               lcall date_time
               sjmp  wait3

down3:
               lcall bounce
               mov   a,r1
               lcall check_bcd_down
               mov   r1,a
               dec   r1
               cjne  r1,#0F9h,ok3
               mov   r1,#99h
               sjmp  ok3


go_back3:
               lcall bounce
               ljmp  set_date


set_hours:                              ; Same as month but for hours
               lcall bounce
               mov   a,#4
               mov   b,#6
               lcall placecur
               lcall prtlcd4
               db    '/',0
               mov   a,#4
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    ':',0
               mov   a,#4
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   r0,#offcur
               lcall wrlcdcom4

wait4:
               jnb   Button2,up4
               jnb   Button3,down4
               jnb   Button4,enter4
               jnb   Button1,go_back4
               sjmp  wait4

enter4:
               lcall bounce
               sjmp  set_minutes

up4:
               lcall bounce
               mov   a,r5
               lcall check_bcd
               mov   r5,a
               cjne  r5,#24h,ok4
               mov   r5,#00h
               sjmp  ok4

ok4:
               lcall date_time
               sjmp  wait4

down4:
               lcall bounce
               mov   a,r5
               lcall check_bcd_down
               mov   r5,a
               dec   r5
               cjne  r5,#0f9h,ok4
               mov   r5,#23h
               sjmp  ok4

go_back4:
               lcall bounce
               ljmp  set_year

set_minutes:                          ; Same as month but for Minutes
               lcall bounce
               mov   a,#4
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#16
               lcall placecur
               lcall prtlcd4
               db    ':',0
               mov   a,#4
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   r0,#offcur
               lcall wrlcdcom4

wait5:
               jnb   Button2,up5
               jnb   Button3,down5
               jnb   Button4,enter5
               jnb   Button1,go_back5
               sjmp  wait5

up5:
               lcall bounce
               mov   a,r6
               lcall check_bcd
               mov   r6,a
               cjne  r6,#60h,ok5
               mov   r6,#00h
               sjmp  ok5

ok5:
               lcall bounce
               lcall date_time
               sjmp  wait5

down5:
               lcall bounce
               mov   a,r6
               lcall check_bcd_down
               mov   r6,a
               dec   r6
               cjne  r6,#0f9h,ok5
               mov   r6,#59h
               sjmp  ok5


go_back5:
               lcall bounce
               ljmp set_hours


enter5:                         ; Dont need to set seconds so zero them

               lcall bounce
               mov   r7,#00h           ; load seconds with 00's


set_it:
               lcall resetlcd4             ; Gives you a choice if you
               mov   a,#2                  ; want to store the data
               mov   b,#0                  ; in the chip, or just
               lcall placecur              ; forget it.
               lcall prtlcd4
               db    ' #2  ACCEPT CHANGES',0
               mov   a,#3
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' #3  IGNORE CHANGES',0
               mov   a,#1
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '___________________',0
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '____________________',0
               mov   r0,#offcur
               lcall wrlcdcom4

wait_set_it:
               jnb   Button2,doit
               jnb   Button3,dont
               jnb   Button4,wait_set_it
               jnb   Button1,wait_set_it
               sjmp  wait_set_it

dont:
               lcall bounce           ; goes back to display time without
               ljmp  msg              ; doing anything

doit:
               lcall bounce
               lcall resetlcd4        ; changes the time data on chip
               mov   a,#2
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' SETTING TIME CLOCK ',0
               lcall big_delay2

               lcall open             ; open the chip
               mov   r0,#8            ; how many bytes
g:             mov   a,@r0            ; move data from address to acc
               lcall wbyte            ; write it to the chip
               dec   r0               ; go to next address
               cjne  r0,#0,g          ; if not all done do it over
               lcall close            ; close the chip
               ljmp  msg              ; go display the time

; ========================================================================
;                Routines to set alarms and controls
;  The setting routines work just like the time setting routine. The only
; differences is where the data is printed on the LCD and where it is
; stored once the data is set.
; ========================================================================
set_alarms:    
               lcall bounce
               lcall resetlcd4        ; sets display so you know
               mov   a,#1             ; which button to push
               mov   b,#4
               lcall placecur
               lcall prtlcd4
               db    ' *** ALARMS *** ',0
               mov   a,#2
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' #2  DAILY ALARMS',0
               mov   a,#3
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' #3  CONTROL STATUS',0
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' #4  CONTROL SET  ',0
               mov   r0,#offcur
               lcall wrlcdcom4

alarm_wait:
               lcall bounce
               jnb   button1,return_alarms
               jnb   button2,daily_alarm
               jnb   button3,con_alarm_stat
               jnb   button4,con_alarm1
               sjmp  alarm_wait

return_alarms:
               lcall bounce
               ljmp  msg
con_alarm1:
               lcall bounce
               ljmp  con_alarm
con_alarm_stat:
               lcall bounce
               ljmp  con_alarm_status
daily_alarm:                        ; sets up display for this routine
               lcall bounce
               lcall resetlcd4
               mov   a,#1
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    ' DAILY ALARM ',0
               mov   a,#2
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '*-               -*',0
               mov   a,#3
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    'ON Time   00:00:00 ',0
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '-------------------',0
               mov   a,#3
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#2
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   r0,#offcur
               lcall wrlcdcom4
               mov   a,#2
               mov   b,#4
               lcall placecur
               mov   r4,#1
               lcall alarm_day

wait_day:                               ; sets the day for the alarm
               jnb   Button2,up_day
               jnb   Button3,down_day
               jnb   Button4,enter_day
               jnb   Button1,return_day
               sjmp  wait_day

enter_day:
               lcall bounce             ; all done go to next data
               mov   10h,r4
               sjmp  alarm_hour

up_day:
               lcall bounce             ; increases number by one
               inc   r4
               cjne  r4,#8h,oka_day
               mov   r4,#1h
               sjmp  oka_day

oka_day:                                ; data is good
               mov   a,#2
               mov   b,#4
               lcall placecur
               lcall alarm_day
               sjmp  wait_day

down_day:
               lcall bounce             ; decreases number by one
               dec   r4
               cjne  r4,#0h,oka_day
               mov   r4,#7h
               mov   a,#2
               mov   b,#4
               lcall placecur
               lcall alarm_day
               sjmp  wait_day

return_day:                           ; button on breaks out of routine
               lcall bounce           ; and goes back to displaying time
               ljmp  set_alarms

alarm_hour:                           ; sets the alarm on hour
               mov   11h,#00
               mov   a,#2
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#3
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    ':',0
               mov   a,#3
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   r0,#offcur
               lcall wrlcdcom4

wait_hour:
               jnb   Button2,up_hour
               jnb   Button3,down_hour
               jnb   Button4,enter_hour
               jnb   Button1,return_hour
               sjmp  wait_hour

up_hour:
               lcall bounce
               mov   a,11h
               lcall check_bcd
               mov   r0,a
               cjne  r0,#24h,ok_hour
               mov   r0,#00h
               sjmp  ok_hour

ok_hour:
               mov   11h,r0
               mov   a,#3
               mov   b,#11
               lcall placecur
               mov   a,11h
               lcall write_bcd
               sjmp  wait_hour

down_hour:
               lcall bounce
               mov   a,11h
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0f9h,ok_hour
               mov   r0,#23h
               sjmp  ok_hour

enter_hour:
               lcall bounce
               sjmp  alarm_minutes

return_hour:
               lcall bounce
               ljmp  daily_alarm

alarm_minutes:                          ; sets the alarm on minutes
               mov   12h,#00
               mov   a,#3
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '-------------------',0
               mov   a,#3
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   r0,#offcur
               lcall wrlcdcom4

wait_min:
               jnb   Button2,up_min
               jnb   Button3,down_min
               jnb   Button4,enter_min
               jnb   Button1,return_min
               sjmp  wait_min

up_min:
               lcall bounce
               mov   a,12h
               lcall check_bcd
               mov   r0,a
               cjne  r0,#60h,ok_min
               mov   r0,#00h
               sjmp  ok_min

ok_min:
               mov   12h,r0
               mov   a,#3
               mov   b,#14
               lcall placecur
               mov   a,12h
               lcall write_bcd
               sjmp  wait_min

down_min:
               lcall bounce
               mov   a,12h
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0f9h,ok_min
               mov   r0,#59h
               sjmp  ok_min

enter_min:
               lcall bounce
               sjmp  alarm_sec

return_min:
               lcall bounce
               ljmp  alarm_hour

alarm_sec:                             ; zeros the seconds
               mov   a,#3
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   13h,#00h
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    '  #2 SET   #3 REDO  ',0 ; asks a question.
wait_set:
               lcall bounce
               jnb   button1,return_sec
               jnb   button2,alarm_set
               jnb   button3,redo_set
               jnb   button4,wait_set
               sjmp  wait_set
return_sec:
               lcall bounce
               ljmp  alarm_minutes
redo_set:
               lcall bounce
               ljmp  daily_alarm
alarm_set:                             ; writes the data to Xmem
               lcall bounce
               mov   a,10h             ; Day number in 10h
               dec   a
               mov   b,#10h
               mul   ab                ; mutiply by 10
               mov   dpl,a             ; LSB of DPTR
               mov   dph,#1eh          ; MSB of DPTR
               mov   a,11h             ; move the hours to acc
               movx  @dptr,a           ; move it to Xmem
               inc   dpl               ; increse LSB of DPTR
               mov   a,12h             ; get minutes
               movx  @dptr,a           ; put data in Xmem
               inc   dpl               ; increse DPTR
               mov   a,13h             ; get Seconds
               movx  @dptr,a           ; put data in Xmem
               ljmp  msg               ; go back to displaying time

; ========================================================================
;        Routine for Displaying Status of Control Outputs
; In the first part of this routine we have a way of showing which
; relays have a time and date set.
; ========================================================================
con_alarm_status:                  ; set up the display to show status
               lcall bounce
               lcall resetlcd4
               mov   a,#1
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    'CONTROL STATUS',0
               mov   a,#2
               mov   b,#2
               lcall placecur
               lcall prtlcd4
               db    'Y=Set  N=Not set',0
               mov   a,#3
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' #1   #2   #3   #4  ',0
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' #5   #6   #7   #8  ',0
               mov   r0,#offcur
               lcall wrlcdcom4

                           ; status is kept in Imem 20h. The first
                           ; bit addressable byte of Imem

one_set:       jnb   00h,one_notset       ; check status bit and show
               mov   a,#3
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    'Y',0                ; if its set
               sjmp  two_set
one_notset:    mov   a,#3
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    'N',0                ; or not set
two_set:       jnb   01h,two_notset
               mov   a,#3
               mov   b,#8
               lcall placecur
               lcall prtlcd4
               db    'Y',0
               sjmp  three_set
two_notset:    mov   a,#3
               mov   b,#8
               lcall placecur
               lcall prtlcd4
               db    'N',0
three_set:     jnb   02h,three_notset
               mov   a,#3
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    'Y',0
               sjmp  four_set
three_notset:  mov   a,#3
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    'N',0
four_set:      jnb   03h,four_notset
               mov   a,#3
               mov   b,#18
               lcall placecur
               lcall prtlcd4
               db    'Y',0
               sjmp  five_set
four_notset:   mov   a,#3
               mov   b,#18
               lcall placecur
               lcall prtlcd4
               db    'N',0
five_set:      jnb   04h,five_notset
               mov   a,#4
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    'Y',0
               sjmp  six_set
five_notset:   mov   a,#4
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    'N',0
six_set:       jnb   05h,six_notset
               mov   a,#4
               mov   b,#8
               lcall placecur
               lcall prtlcd4
               db    'Y',0
               sjmp  seven_set
six_notset:    mov   a,#4
               mov   b,#8
               lcall placecur
               lcall prtlcd4
               db    'N',0
seven_set:     jnb   06h,seven_notset
               mov   a,#4
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    'Y',0
               sjmp  eight_set
seven_notset:  mov   a,#4
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    'N',0
eight_set:     jnb   07h,eight_notset
               mov   a,#4
               mov   b,#18
               lcall placecur
               lcall prtlcd4
               db    'Y',0
               sjmp  done
eight_notset:  mov   a,#4
               mov   b,#18
               lcall placecur
               lcall prtlcd4
               db    'N',0
done:          mov   r0,#offcur
               lcall wrlcdcom4
               mov   10h,#01h
               mov   11h,#00h
               mov   12h,#00h
               sjmp  ok_status

stay:
               jnb   button1,back_status
               jnb   button2,up_status
               jnb   button3,down_status
               jnb   button4,enter_status
               sjmp  stay

back_status:
               lcall bounce
               ljmp  set_alarms
up_status:
               lcall bounce
               mov   a,10h
               inc   a
               mov   10h,a
               cjne  a,#09h,ok_status
               mov   10h,#01h
               sjmp  ok_status

down_status:
               lcall bounce
               mov   a,10h
               dec   a
               mov   10h,a
               cjne  a,#00h,ok_status
               mov   10h,#08h
               sjmp  ok_status

ok_status:
               mov   a,11h
               mov   b,12h
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,10h
               clr   c
               add   a,#0fbh
               jc    forth_line
               mov   b,#05h
               mov   a,10h
               dec   a
               mul   ab
               mov   b,a
               mov   a,#3h
               sjmp  show_arrow
forth_line:
               mov   b,#05h
               mov   a,10h
               clr   c
               subb  a,#5h
               mul   ab
               mov   b,a
               mov   a,#4h

show_arrow:
               mov   11h,a
               mov   12h,b
               lcall placecur
               lcall prtlcd4
               db    126,0                  ; put arrow by relay #
               mov   r0,#offcur             ; turn off cursor
               lcall wrlcdcom4
               sjmp  stay

enter_status:
               lcall bounce
               lcall get_status
               lcall show_status

stay_enter:
               jnb   button1,con_alarm_status1
               jnb   button1,con_alarm_status1
               jnb   button1,con_alarm_status1
               jnb   button1,con_alarm_status1
               ljmp  stay_enter

con_alarm_status1:
               ljmp  con_alarm_status

set_con_alarm2:
               lcall bounce
               ljmp  con_alarm

show_status:
               lcall resetlcd4
               mov   a,#1
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    'CONTROL STATUS #',0
               mov   a,#2
               mov   b,#4
               lcall placecur
               lcall prtlcd4
               db    'ON',0
               mov   a,#2
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    'OFF',0
               mov   a,#3
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    '  /  /  ',0
               mov   a,#3
               mov   b,#11
               lcall placecur
               lcall prtlcd4
               db    '  /  /  ',0
               mov   a,#4
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    '00:00:00',0
               mov   a,#4
               mov   b,#11
               lcall placecur
               lcall prtlcd4
               db    '00:00:00',0
               mov   a,#1
               mov   b,#17
               lcall placecur
               mov   a,10h
               lcall write_bcd
               mov   a,#3
               mov   b,#1
               lcall placecur
               mov   a,11h
               lcall write_bcd
               mov   a,#3
               mov   b,#4
               lcall placecur
               mov   a,12h
               lcall write_bcd
               mov   a,#3
               mov   b,#7
               lcall placecur
               mov   a,13h
               lcall write_bcd
               mov   a,#4
               mov   b,#1
               lcall placecur
               mov   a,14h
               lcall write_bcd
               mov   a,#4
               mov   b,#4
               lcall placecur
               mov   a,15h
               lcall write_bcd
               mov   a,#3
               mov   b,#11
               lcall placecur
               mov   a,16h
               lcall write_bcd
               mov   a,#3
               mov   b,#14
               lcall placecur
               mov   a,17h
               lcall write_bcd
               mov   a,#3
               mov   b,#17
               lcall placecur
               mov   a,18h
               lcall write_bcd
               mov   a,#4
               mov   b,#11
               lcall placecur
               mov   a,19h
               lcall write_bcd
               mov   a,#4
               mov   b,#14
               lcall placecur
               mov   a,1ah
               lcall write_bcd
               mov   r0,#offcur
               lcall wrlcdcom4
               ret


get_status:
               mov   a,10h          ; get relay number
               mov   b,#10h         ; move 10 to B register
               mul   ab             ; multiply by 10
               mov   dpl,a          ; LSB of DPTR is 10*relay number
               mov   dph,#1ch       ; set MSB of DPTR
               mov   r0,#11h        ; set starting point of loop
another:       movx  a,@dptr        ; get data from Xmem
               mov   @r0,a          ; move it to Imem
               inc   dpl            ; go to the next Xmem address
               inc   r0             ; go to next Imem address
               cjne  r0,#1bh,another ; If not all of them do it again
               ret

con_alarm:                          ; setup display to set ON data
               lcall bounce
               lcall resetlcd4
               mov   a,#1
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    'CONTROL ACTIVATION',0
               mov   a,#3
               mov   b,#2
               lcall placecur
               lcall prtlcd4
               db    '=DATE=',0
               mov   a,#3
               mov   b,#12
               lcall placecur
               lcall prtlcd4
               db    '=TIME=',0
               mov   a,#4
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    '--/--/--',0
               mov   a,#4
               mov   b,#11
               lcall placecur
               lcall prtlcd4
               db    '00:00:00',0
               mov   a,#2
               mov   b,#2
               lcall placecur
               lcall prtlcd4
               db    'RELAY #',0
               mov   a,#2
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    126,0                    ; put arrow by relay #
               mov   r0,#offcur
               lcall wrlcdcom4
               mov   10h,#01h                 ; set for first relay
               mov   a,#2                     ; row to display it
               mov   b,#9                     ; colum to display it
               lcall placecur
               mov   a,10h                    ; put data in acc
               lcall write_bcd                ; send it to display

relay_wait:                                   ; buttons
               jnb   button1,return_relay
               jnb   button2,up_relay
               jnb   button3,down_relay
               jnb   button4,enter_relay
               sjmp  relay_wait

up_relay:                                     ; increase by one
               lcall bounce
               mov   a,10h
               lcall check_bcd
               mov   r0,a
               cjne  r0,#09h,ok_relay
               mov   r0,#01h
               sjmp  ok_relay

ok_relay:                                     ; data is allright
               mov   10h,r0
               mov   a,#2
               mov   b,#9
               lcall placecur
               mov   a,10h
               lcall write_bcd
               sjmp  relay_wait

down_relay:                                    ; decrease by one
               lcall bounce
               mov   a,10h
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0h,ok_relay
               mov   r0,#08h
               sjmp  ok_relay

enter_relay:                                   ; data is what we want
               lcall bounce
               sjmp  on_month                  ; go set the next one

return_relay:
               lcall bounce
               ljmp  msg

on_month:
               mov   a,#2
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   11h,r2
               mov   a,#4
               mov   b,#1
               lcall placecur
               mov   a,11h
               lcall write_bcd

wait_on_month:
               jnb   button1,return_on_month
               jnb   button2,up_on_month
               jnb   button3,down_on_month
               jnb   button4,enter_on_month
               sjmp  wait_on_month

up_on_month:
               lcall bounce
               mov   a,11h
               lcall check_bcd
               mov   r0,a
               cjne  r0,#13h,ok_on_month
               mov   r0,#01h
               sjmp  ok_on_month

ok_on_month:
               mov   11h,r0
               mov   a,#4
               mov   b,#1
               lcall placecur
               mov   a,11h
               lcall write_bcd
               sjmp  wait_on_month

down_on_month:
               lcall bounce
               mov   a,11h
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0h,ok_on_month
               mov   r0,#12h
               sjmp  ok_on_month

enter_on_month:
               lcall bounce
               sjmp  on_date

return_on_month:
               lcall bounce
               ljmp  msg


on_date:
               mov   a,#4
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   12h,r3
               mov   a,#4
               mov   b,#4
               lcall placecur
               mov   a,12h
               lcall write_bcd

wait_on_date:
               jnb   button1,return_on_date
               jnb   button2,up_on_date
               jnb   button3,down_on_date
               jnb   button4,enter_on_date
               sjmp  wait_on_date

up_on_date:
               lcall bounce
               mov   a,12h
               lcall check_bcd
               mov   r0,a
               cjne  r0,#32h,ok_on_date
               mov   r0,#01h
               sjmp  ok_on_date

ok_on_date:
               mov   12h,r0
               mov   a,#4
               mov   b,#4
               lcall placecur
               mov   a,12h
               lcall write_bcd
               sjmp  wait_on_date

down_on_date:
               lcall bounce
               mov   a,12h
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0h,ok_on_date
               mov   r0,#31h
               sjmp  ok_on_date

enter_on_date:
               lcall bounce
               sjmp  on_year

return_on_date:
               lcall bounce
               ljmp  msg

on_year:
               mov   a,#4
               mov   b,#3
               lcall placecur
               lcall prtlcd4
               db    '/',0
               mov   a,#4
               mov   b,#6
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   13h,r1
               mov   a,#4
               mov   b,#7
               lcall placecur
               mov   a,13h
               lcall write_bcd

wait_on_year:
               jnb   button1,return_on_year
               jnb   button2,up_on_year
               jnb   button3,down_on_year
               jnb   button4,enter_on_year
               sjmp  wait_on_year

up_on_year:
               lcall bounce
               mov   a,13h
               lcall check_bcd
               mov   r0,a
               cjne  r0,#09ah,ok_on_year
               mov   r0,#00h
               sjmp  ok_on_year

ok_on_year:
               mov   13h,r0
               mov   a,#4
               mov   b,#7
               lcall placecur
               mov   a,13h
               lcall write_bcd
               sjmp  wait_on_year

down_on_year:
               lcall bounce
               mov   a,13h
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0f9h,ok_on_year
               mov   r0,#99h
               sjmp  ok_on_year

enter_on_year:
               lcall bounce
               sjmp  on_hour

return_on_year:
               lcall bounce
               ljmp  msg

on_hour:
               mov   a,#4
               mov   b,#6
               lcall placecur
               lcall prtlcd4
               db    '/',0
               mov   a,#4
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   14h,r5
               mov   a,#4
               mov   b,#11
               lcall placecur
               mov   a,14h
               lcall write_bcd

wait_on_hour:
               jnb   button1,return_on_hour
               jnb   button2,up_on_hour
               jnb   button3,down_on_hour
               jnb   button4,enter_on_hour
               sjmp  wait_on_hour

up_on_hour:
               lcall bounce
               mov   a,14h
               lcall check_bcd
               mov   r0,a
               cjne  r0,#24h,ok_on_hour
               mov   r0,#00h
               sjmp  ok_on_hour

ok_on_hour:
               mov   14h,r0
               mov   a,#4
               mov   b,#11
               lcall placecur
               mov   a,14h
               lcall write_bcd
               sjmp  wait_on_hour

down_on_hour:
               lcall bounce
               mov   a,14h
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0f9h,ok_on_hour
               mov   r0,#23h
               sjmp  ok_on_hour

enter_on_hour:
               lcall bounce
               sjmp  on_min

return_on_hour:
               lcall bounce
               ljmp  msg

on_min:
               mov   a,#4
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#13
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   15h,r6
               mov   a,#4
               mov   b,#14
               lcall placecur
               mov   a,15h
               lcall write_bcd

wait_on_min:
               jnb   button1,return_on_min
               jnb   button2,up_on_min
               jnb   button3,down_on_min
               jnb   button4,enter_on_min
               sjmp  wait_on_min

up_on_min:
               lcall bounce
               mov   a,15h
               lcall check_bcd
               mov   r0,a
               cjne  r0,#60h,ok_on_min
               mov   r0,#00h
               sjmp  ok_on_min

ok_on_min:
               mov   15h,r0
               mov   a,#4
               mov   b,#14
               lcall placecur
               mov   a,15h
               lcall write_bcd
               sjmp  wait_on_min

down_on_min:
               lcall bounce
               mov   a,15h
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0f9h,ok_on_min
               mov   r0,#59h
               sjmp  ok_on_min

enter_on_min:
               lcall bounce
               sjmp  dur_days

return_on_min:
               lcall bounce
               ljmp  msg
; ========================================================================
;     At this point the date and time for the control to come on is set
; and stored in Imem. Now the display changes and the amount of duration
; for the control is set.
; ========================================================================
dur_days:                           ; set up display for the duration
               lcall resetlcd4
               mov   a,#1
               mov   b,#2
               lcall placecur
               lcall prtlcd4
               db    'CONTROL DURATION',0
               mov   a,#3
               mov   b,#1
               lcall placecur
               lcall prtlcd4
               db    'DAYS=',0
               mov   a,#2
               mov   b,#5
               lcall placecur
               lcall prtlcd4
               db    'RELAY #',0
               mov   a,#3
               mov   b,#11
               lcall placecur
               lcall prtlcd4
               db    'HOURS=',0
               mov   a,#4
               mov   b,#5
               lcall placecur
               lcall prtlcd4
               db    'MINUTES=',0
               mov   a,#3
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   r0,#offcur
               lcall wrlcdcoM4
               mov   a,#2
               mov   b,#12
               lcall placecur
               mov   a,10h
               lcall write_bcd
               mov   1bh,#00h
               mov   a,#3
               mov   b,#6
               lcall placecur
               mov   a,1bh
               lcall write_bcd

wait_dur_days:
               jnb   button1,return_dur_days
               jnb   button2,up_dur_days
               jnb   button3,down_dur_days
               jnb   button4,enter_dur_days
               sjmp  wait_dur_days

up_dur_days:
               lcall bounce
               mov   a,1bh
               lcall check_bcd
               mov   r0,a
               cjne  r0,#32h,ok_dur_days
               mov   r0,#00h
               sjmp  ok_dur_days

ok_dur_days:
               mov   1bh,r0
               mov   a,#3
               mov   b,#6
               lcall placecur
               mov   a,1bh
               lcall write_bcd
               sjmp  wait_dur_days

down_dur_days:
               lcall bounce
               mov   a,1bh
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0f9h,ok_dur_days
               mov   r0,#31h
               sjmp  ok_dur_days

enter_dur_days:
               lcall bounce
               sjmp  dur_hours

return_dur_days:
               lcall bounce
               ljmp  msg

dur_hours:
               mov   a,#3
               mov   b,#0
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#3
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   1ch,#00h
               mov   a,#3
               mov   b,#17
               lcall placecur
               mov   a,1ch
               lcall write_bcd

wait_dur_hours:
               jnb   button1,return_dur_hours
               jnb   button2,up_dur_hours
               jnb   button3,down_dur_hours
               jnb   button4,enter_dur_hours
               sjmp  wait_dur_hours

up_dur_hours:
               lcall bounce
               mov   a,1ch
               lcall check_bcd
               mov   r0,a
               cjne  r0,#24h,ok_dur_hours
               mov   r0,#00h
               sjmp  ok_dur_hours

ok_dur_hours:
               mov   1ch,r0
               mov   a,#3
               mov   b,#17
               lcall placecur
               mov   a,1ch
               lcall write_bcd
               sjmp  wait_dur_hours

down_dur_hours:
               lcall bounce
               mov   a,1ch
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0f9h,ok_dur_hours
               mov   r0,#23h
               sjmp  ok_dur_hours

enter_dur_hours:
               lcall bounce
               sjmp  dur_mins

return_dur_hours:
               lcall bounce
               ljmp  msg

dur_mins:
               mov   a,#3
               mov   b,#10
               lcall placecur
               lcall prtlcd4
               db    ' ',0
               mov   a,#4
               mov   b,#4
               lcall placecur
               lcall prtlcd4
               db    126,0
               mov   1dh,#00h
               mov   a,#4
               mov   b,#13
               lcall placecur
               mov   a,1dh
               lcall write_bcd

wait_dur_mins:
               jnb   button1,return_dur_mins
               jnb   button2,up_dur_mins
               jnb   button3,down_dur_mins
               jnb   button4,enter_dur_mins
               sjmp  wait_dur_mins

up_dur_mins:
               lcall bounce
               mov   a,1dh
               lcall check_bcd
               mov   r0,a
               cjne  r0,#60h,ok_dur_mins
               mov   r0,#00h
               sjmp  ok_dur_mins

ok_dur_mins:
               mov   1dh,r0
               mov   a,#4
               mov   b,#13
               lcall placecur
               mov   a,1dh
               lcall write_bcd
               sjmp  wait_dur_mins

down_dur_mins:
               lcall bounce
               mov   a,1dh
               lcall check_bcd_down
               mov   r0,a
               dec   r0
               cjne  r0,#0f9h,ok_dur_mins
               mov   r0,#59h
               sjmp  ok_dur_mins

enter_dur_mins:
               lcall bounce
               sjmp  figure_off_data

return_dur_mins:
               lcall bounce
               ljmp  msg


figure_off_data:

; ========================================================================
;         first we add the duration minutes to the ON time minutes.
; ========================================================================
minutes:
               mov   16h,#0               ; clear the registers
               mov   17h,#0
               mov   18h,#0
               mov   19h,#0
               mov   1ah,#0
               mov   a,15h                ; ON Minutes
               anl   a,#0fh               ; get rid of upper bits
               cjne  a,#00h,nozero1       ; if it ends in 0 we have to
               sjmp  zero                 ; adjust it

nozero1:       mov   a,1dh                ; duration minutes
               anl   a,#0fh               ; same as for on minutes
               cjne  a,#00,nozero2
               sjmp  zero

nozero2:       mov   a,15h                ; on minutes
               mov   b,1dh                ; duration minutes
               clr   c                    ; clear the carry flag
               add   a,b                  ; add them together
               mov   r0,a                 ; save unadjusted sum
               clr   c                    ; clear the carry flag
               add   a,#0a6h              ; check to see if sum is over 60
               jc    add_hour             ; if so we increse the hour
under_hour:    mov   a,r0                 ; it isnt so get sum
               da    a                    ; adjust it
               mov   1ah,r0               ; put data in memory
               sjmp  hours                ; goto set duration hours

zero:
               mov   a,15h                ; on minutes
               mov   b,1dh                ; duration minutes
               clr   c                    ; clear carry flag
               add   a,b                  ; add them together
               clr   c                    ; clear carry flag
               mov   r0,a                 ; save sum
               clr   c                    ; clear carry flag
               add   a,#0a6h              ; check to see if over 60
               jc    add_hour2            ; it is so add a hour
               sjmp  under_hour           ; it isnt so go on and save it

; if the adjusted number is larger then 60 minutes we increase duration
; hours by 1

add_hour:
               mov   a,r0           ; get sum
               anl   a,#0fh         ; remove upper bits
               cjne  a,#00h,reg     ; if it doesnt end in zero go reg
add_hour2:     mov   a,r0           ; it ends in zero so get number
               clr   c              ; clear the carry flag
               subb  a,#60h         ; subtract 60 hex from it
               mov   1ah,a          ; you end up with a good number
               mov   a,1ch          ; get hour
               clr   c              ; clear carry flag
               add   a,#1           ; add one to hour
               da    a              ; adjust it
               mov   1ch,a          ; save it to Imem
               sjmp  hours          ; go to hours
reg:
               mov   a,r0           ; get number
               clr   c              ; clear carry flag
               subb  a,#5ah         ; subtract a hour from it
               mov   1ah,a          ; save it in Imem
               mov   a,1ch          ; get the duration hour
               clr   c              ; clear carry flag
               add   a,#1           ; add one to it
               da    a              ; adjust it
               mov   1ch,a          ; save it to Imem

; Then we add duration hours to on time hours

hours:
               mov   a,1ch          ; duration hours
               mov   b,14h          ; on hours
               clr   c              ; clear carry flag
               add   a,b            ; add them together
               da    a              ; adjust them
               mov   r0,a           ; temporary save
               clr   c              ; clear the carry flag
               add   a,#0dch        ; add DC hex
               jc    add_day        ; if it over 24 add a day
               mov   19h,r0         ; if not just save it to Imem
               sjmp  date           ; go on

; if more the 24 hours we increase the duration day by 1

add_day:
               mov   a,r0           ; get adjusted sum
               clr   c
               subb  a,#24h         ; remove a day
               clr   c
               add   a,#1           ; get ready to adjust
               da    a              ; adjust it
               mov   19h,a          ; save it to Imem
               mov   a,1bh          ; get duration days
               clr   c
               add   a,#1h          ; add one to them
               da    a              ; adjust it
               mov   1bh,a          ; save it in Imem

; then we add the duration days to the on date

date:
               mov   a,1bh          ; durations days
               mov   b,12h          ; on date
               add   a,b            ; add them together
               da    a              ; adjust it
               mov   r0,a           ; temporary save
               mov   a,11h          ; get the month
               cjne  a,#02h,not_feb ; Is it february?
               clr   c              ; clear the carry flag
               mov   a,r0           ; get the sum
               add   a,#0d8h        ; see if its over 28
               jc    add_month      ; if so add a month
               mov   17h,r0         ; save it to Imem
               ljmp  months         ; go on

; if its Feburary and there is more then 28 days then increase duration
; months by 1
             
add_month:
               mov   a,r0           ; get the sum
               subb  a,#28h         ; subtract a month of Feb.
               clr   c              ; clear the carry flag
               mov   r0,a           ; temporary save
               add   a,#0f6h        ; check if its over 10
               jc    opps           ; if it is adjust it
               mov   a,r0           ; get original number
back:          add   a,#1           ; get ready to adjust
               da    a              ; adjust it
               mov   17h,a          ; save it to Imem
               mov   a,1eh          ; get duration months
               add   a,#1           ; add one to it
               da    a              ; adjust it
               mov   1eh,a          ; save it to Imem
               ljmp  months         ; go on
opps:
               mov   a,r0           ; get the number
               subb  a,#5h          ; subtract 5 so its a legal BCD
               sjmp  back           ; go back and treat it like the others

            ; From here down we just figure out how long of a month it is
not_feb:
               mov    a,11h
               cjne   a,#1h,not_jan
               ljmp   month_31

not_jan:
               mov    a,11h
               cjne   a,#3h,not_mar
               ljmp   month_31

not_mar:
               mov    a,11h
               cjne   a,#4h,not_apr
               ljmp   month_30

not_apr:
               mov    a,11h
               cjne   a,#5h,not_may
               ljmp   month_31

not_may:
               mov    a,11h
               cjne   a,#6h,not_jun
               ljmp   month_30

not_jun:
               mov    a,11h
               cjne   a,#7h,not_jul
               ljmp   month_31

not_jul:
               mov    a,11h
               cjne   a,#8h,not_aug
               ljmp   month_31

not_aug:
               mov    a,11h
               cjne   a,#9h,not_sep
               ljmp   month_30

not_sep:
               mov    a,11h
               cjne   a,#10h,not_oct
               ljmp   month_31

not_oct:
               mov   a,11h
               cjne  a,#11h,not_nov
               ljmp  month_30

not_nov:
               mov   a,11h
               cjne  a,#12h,dont_no
               ljmp  month_31

dont_no:
               ljmp  date

; if its a month with 30 days and there are over 30 days increase
; duration months by 1

month_30:
               clr   c              ; clear carry flag
               mov   a,r0           ; temporary save
               add   a,#0cfh        ; check to see if over 30
               jc    add_month_30   ; if it is add a month
               mov   17h,r0         ; it isn't so save to Imem
               sjmp  months         ; go on
add_month_30:
               mov   a,r0           ; get original number
               subb  a,#30h         ; subtract a month
               clr   c              ; clear the carry flag
               mov   r0,a           ; temporary save
               add   a,#0f6h        ; check to see if its BCD
               jc    opps_30        ; it ain't so fix it
               mov   a,r0           ; the number back
back_30:       add   a,#1           ; get it ready to adjust
               da    a              ; adjust it
               mov   17h,a          ; save it to Imem
               mov   a,1eh          ; get the duration months
               add   a,#1h          ; add on to it
               da    a              ; adjust it
               mov   1eh,a          ; save it to Imem
               sjmp  months         ; go on
opps_30:
               mov   a,r0           ; get the number
               subb  a,#5h          ; make it BCD
               sjmp  back_30        ; go back


; if its a month with 31 days and there are over 31 days then increase
; duration months by 1

month_31:      clr   c            ; this works like the one above
               mov   a,r0         ; except for 31 days
               add   a,#0ceh
               jc    add_month_31
               mov   17h,r0
               sjmp  months

add_month_31:
               mov   a,r0
               subb  a,#31h
               clr   c
               mov   r0,a
               add   a,#0f6h
               jc    opps_31
               mov   a,r0
back_31:       add   a,#1
               da    a
               mov   17h,a
               mov   a,1eh
               add   a,#1h
               da    a
               mov   1eh,a
               ajmp  months
opps_31:
               mov    a,r0
               subb   a,#5h
               sjmp   back_31
months:
               mov   a,1eh          ; duration months
               mov   b,11h          ; on month
               add   a,b            ; add them together
               da    a              ; adjust it
               mov   r0,a           ; temporary save
               clr   c              ; clear carry flag
               add   a,#0f6h        ; check to see if over 12
               jc    add_year       ; if it is add a year
               mov   16h,r0         ; save it to Imem
               mov   18h,13h        ; make duration year same as on year
               sjmp  store_alarm    ; go on

add_year:
               mov   a,r0           ; get original sum
               subb  a,#12h         ; subtract 12 from it
               add   a,#1           ; get it ready to adjust
               da    a              ; adjust it
               mov   16h,a          ; save it to Imem
               mov   a,13h          ; get on year
               add   a,#1           ; add one to it
               da    a              ; adjust it
               mov   18h,a          ; save it to Imem

store_alarm:
               mov   a,10h          ; get relay number
               cjne  a,#8h,nextA    ; if its not #8 go on
               mov   r0,#80h        ; if it is #8 the set data
               sjmp  orem          ; go to add it to status byte
nexta:
               mov   a,10h
               cjne  a,#7h,nextB    ; do it for #7
               mov   r0,#40h
               sjmp  orem
nextb:
               mov   a,10h
               cjne  a,#6h,nextC    ; do it for #6
               mov   r0,#20h
               sjmp  orem
nextc:
               mov   a,10h
               cjne  a,#5h,nextD    ; do it for #5
               mov   r0,#10h
               sjmp  orem
nextd:
               mov   a,10h
               cjne  a,#4h,nextE    ; do it for #4
               mov   r0,#08h
               sjmp  orem
nexte:
               mov   a,10h
               cjne  a,#3h,nextF    ; do it for #3
               mov   r0,#04h
               sjmp  orem
nextf:
               mov   a,10h
               cjne  a,#2h,nextG    ; do it for #2
               mov   r0,#02h
               sjmp  orem
nextg:
               mov   a,10h
               cjne  a,#1h,nextH    ; do it for #1
               mov   r0,#01h
               sjmp  orem
nexth:
               sjmp  store_alarm    ; go back and try again

orem:
               mov   b,20h          ; get status byte
               mov   a,r0           ; get data
               orl   a,b            ; OR them together
               mov   dpl,#00h       ; set LSB of DPTR
               mov   dph,#1ch       ; set MSB of DPTR
               movx  @dptr,a        ; move status to Xmem

               mov   a,10h          ; get relay number
               mov   b,#10h         ; move 10 to B register
               mul   ab             ; multiply by 10
               mov   dpl,a          ; LSB of DPTR is 10*relay number
               mov   dph,#1ch       ; set MSB of DPTR
               mov   r0,#11h        ; set starting point of loop
next_one:      mov   a,@r0          ; get what is at address of Imem
               movx  @dptr,a        ; move it to Xmem
               inc   dpl            ; go to the next Xmem address
               inc   r0             ; go to next Imem address
               cjne  r0,#20h,next_one  ; If not all of them do it again
               ljmp  msg            ; go back to displaying the time

; ========================================================================
;                   Button Debouncer delay routine
; Simple delay routine. Just a bunch of loops to waste time. It switches
; the register banks so the other registers aren't changed.
; ========================================================================
bounce:
               mov   psw,#08h      ; use second bank of registers
               mov   r0,#2h        ; how many loops
x4b:
               mov   r2,#0ffh      ; how many of loops inside loops
x5b:
               mov   r5,#0ffh      ; length of each loop
x6b:
               djnz  r5,x6b        ; stay here for awhile
               djnz  r2,x5b        ; loop back up so many times
               djnz  r0,x4b        ; loop to do all the loops
               mov   psw,#00h      ; change back to first register bank
               ret                 ; go back

; ========================================================================
;   Checks to see if the number in Acc is legal BCD
;   if it isnt it will adjust it so it is
; ========================================================================
check_bcd:
         mov      r0,a          ; saves the number to r0
         clr      c             ; clears carry bit
         add      a,#67h        ; checks to see if equal 99
         jc       zeroit        ; if equal then becomes 0
         mov      a,r0          ; gets original number
         anl      a,#0fh        ; gets rid of upper 4 bits
         clr      c             ; clears carry bit
         add      a,#0f7h       ; checks to see if number ends in 9
         jc       adjust        ; if larger adjust it
         mov      a,r0          ; get original number
         inc      a             ; its under 9 and not 0 so increase
         ret                    ; by 1 and go back.

; ========================================================================
; This sub-routine simple takes the BCD number 9 and adds one to it.
; For example BCD 9 is 00001001 and BCD 10 is 00010000, You can't do a
; regular inc. or you get a non-BCD number. What I have done is look
; to see if I have a 9 in the lower four bits and if I do, I take the
; upper four bits and treat it like a regular binary number. I add one to
; it and, Then move it back to the upper four bits. So 9 then becomes 10,
; 19 becomes 20 etc.
; ========================================================================
adjust:
	 mov	  a,r0		; get original number
         swap     a             ; change upper bits to lower bits
         anl      a,#0fh        ; get rid of what was lower bits
         inc      a             ; increase by one
         swap     a             ; switch the upper and lower bits back
         ret                    ; go back with new number

zeroit:                           ; if number is zero send back a zero
	 mov	  a,#00000000b
	 ret
; ========================================================================
;    This routine checks to see if the lower 4 bits are equal to zero.
; If they are then a straight decrease can be done. if not then the number
; has to be converted from a BCD number to a binary number.
; ========================================================================
check_bcd_down:
         mov      r0,a                    ; save original number
         anl      a,#0fh                  ; get rid of upper four bits
	 cjne	  a,#00000000b,go_on_back ;do lower bits =0 ?
         mov      a,r0                    ; get original number
         subb     a,#06h                  ; convert it to a binary number
	 ret

; ========================================================================
;    if the number ends in number less than 9 then just
;    go back and, do a regular decrease on it.
; ========================================================================
go_on_back:
         mov      a,r0                   ; get original number
         ret                             ; go back


; ========================================================================
;                           Routine Write BCD
; This routine changes the BCD number to ASCII format so you can print
; it to the LCD.
; ========================================================================
write_bcd:
               push  acc            ; Save to stack
               swap  a              ; swap lower and upper bits around
               anl   a,#0fh         ; get rid of upper bits
               orl   a,#30h         ; add 30 hex to it
                                    ; makes it a ASCII format
               lcall find           ; go to the look up table
               pop   acc            ; get the number back
               anl   a,#0fh         ; get rid of upper bits
               orl   a,#30h         ; make it ASCII

find:
               sjmp  look_up

st:
               ret

look_up:
               cjne  a,#'0',m1 ; Look-Up routine for Printing to LCD
               lcall prtlcd4
               db       '0',0
               ljmp  st

m1:
               cjne  a,#'1',m2
               lcall prtlcd4
               db       '1',0
               ljmp  st

m2:
               cjne  a,#'2',m3
               lcall prtlcd4
               db       '2',0
               ljmp  st

m3:
               cjne  a,#'3',m4
               lcall prtlcd4
               db       '3',0
               ljmp  st

m4:
               cjne  a,#'4',m5
               lcall prtlcd4
               db       '4',0
               ljmp  st

m5:
               cjne  a,#'5',m6
               lcall prtlcd4
               db       '5',0
               ljmp  st

m6:
               cjne  a,#'6',m7
               lcall prtlcd4
               db       '6',0
               ljmp  st

m7:
               cjne  a,#'7',m8
               lcall prtlcd4
               db       '7',0
               ljmp  st

m8:
               cjne  a,#'8',m9
               lcall prtlcd4
               db       '8',0
               ljmp  st

m9:
               cjne  a,#'9',ok
               lcall prtlcd4
               db       '9',0
               ljmp  st

ok:
               ljmp  st


; ========================================================================
; subroutine placeCur
; this routine sets the cursor position.  the cursor position
; is placed in the b register.  the cursor position starts
; at 0.  the accumulator gives the line number.
; input  : accumulator indicates line number (1, 2)
;        : b register holds the cursor position
; output : none
; ========================================================================
placecur:
               dec   acc     ; this routine will place the cursor at a
               jnz   line2   ; specific place : A holds the line, and
               mov   a,b     ; B holds the column locations when this
                             ; routine is called
               add   a,#80h  ; sets command for line 1
               sjmp  setcur

line2:
               cjne  a,#1,line3
               mov   a,b
               add   a,#0c0h   ; sets location for line 2
               sjmp  setcur

line3:
               cjne  a,#2,line4
               mov   a,b
               add   a,#94h    ; sets location for line 3
               sjmp  setcur

line4:
               mov   a,b
               add   a,#0d4h   ; sets location for line 4

setcur:
               mov   r0,a       ; gets command
               lcall wrlcdcom4  ; sends it to LCD
               ret

; ========================================================================
; subroutine prtLCD4
; takes the string immediately following the call and
; displays on the LCD.  the string is read with the
;   mov a, @a+dptr
; instruction.  thus, the string is in data memory.  since
; the string immediately follows code, the microcontroller
; is used with overlapping code and data memories (the von
; Newman architecture).  string must be terminated with a
; null (0).  upon termination this routine jumps to the
; instruction immediately following the string.
;
; input    : none
; output   : none
; destroys : acc, dptr
; ========================================================================
;
prtLCD4:
         pop   dph               ; pop return address into dptr
         pop   dpl
prtNext:
         clr  a                  ; set offset = 0
         movc a,@a+dptr          ; get chr from code memory
         cjne a,#0,chrOK         ; if chr = 0 then return
         sjmp retPrtLCD4
chrOK:
         mov   r0,a
         lcall wrLCDdata4        ; send character
         inc   dptr              ; point at next character
         ljmp  prtNext           ; loop till end of string
retPrtLCD4:
         mov   a,#1h             ; point to instruction after string
         jmp   @a+dptr           ; return with a jump instruction


; ========================================================================
; subroutine wrLCDdata4
; write a data word to the LCD
;
; data must be placed in r0 by calling program
; ========================================================================
wrLCDdata4:
    clr   LCD_E
    setb  LCD_RS                 ; select send Data
    clr   LCD_RW                 ; select write operation
    push  acc                    ; save accumulator
    mov   a,r0                  ; put data byte in acc

    mov   c, acc.4               ; load high nibble on data bus
    mov   LCD_DB4, c             ; one bit at a time using...
    lcall mdelay
    mov   c, acc.5               ; bit move operations
    mov   LCD_DB5, c
    lcall mdelay                          ; delays are for slow LCDs
    mov   c, acc.6
    mov   LCD_DB6, c
    lcall mdelay                          ; they can be removed
    mov   c, acc.7
    mov   LCD_DB7, c
    lcall mdelay                          ; if this routine is to slow

    setb  LCD_E                  ; pulse the Enable line
    lcall mdelay
    clr   LCD_E

    mov   c, acc.0               ; similarly, load low nibble
    mov   LCD_DB4, c
    lcall mdelay
    mov   c, acc.1
    mov   LCD_DB5, c
    lcall mdelay
    mov   c, acc.2
    mov   LCD_DB6, c
    lcall mdelay
    mov   c, acc.3
    mov   LCD_DB7, c
    lcall mdelay

    lcall pulseEwait4            ; pulse the Enable line...
                                 ; and wait for Busy Flag to clear
    pop   acc
    ret

; ========================================================================
; subroutine wrLCDcom4
; write a command word to the LCD
;
; command must be placed in r0 by calling program
; ========================================================================
wrLCDcom4:
    clr   LCD_E
    clr   LCD_RS                 ; select send Command
    clr   LCD_RW                 ; select Write operation
    push  acc                    ; save accumulator
    mov   a,r0                   ; put data byte in acc

    mov   c, acc.4               ; load high nibble on data bus
    mov   LCD_DB4, c             ; one bit at a time using...
    lcall mdelay
    mov   c, acc.5               ; bit move operatoins
    mov   LCD_DB5, c
    lcall mdelay
    mov   c, acc.6
    mov   LCD_DB6, c
    lcall mdelay
    mov   c, acc.7
    mov   LCD_DB7, c
    lcall mdelay

    setb  LCD_E                  ; pulse the Enable line
    lcall mdelay
    clr   LCD_E

    mov   c, acc.0               ; similarly, load low nibble
    mov   LCD_DB4, c
    lcall mdelay
    mov   c, acc.1
    mov   LCD_DB5, c
    lcall mdelay
    mov   c, acc.2
    mov   LCD_DB6, c
    lcall mdelay
    mov   c, acc.3
    mov   LCD_DB7, c
    lcall mdelay

    lcall pulseEwait4            ; pulse the Enable line...
                                 ; and wait for Busy Flag to clear
    pop   acc
    ret

; ========================================================================
; subroutine pulseEwait4
; generates a positive pulse on the LCD enable line.
; waits for the Busy Flag to clear before returning.
; input    : none
; output   : none
; destroys : LCD_RW, LCD_RS, LCD_DATA
;
; ========================================================================
pulseEwait4:
    clr   LCD_E
    setb  LCD_E                  ; pulse the Enable line
    clr   LCD_E

    mov   LCD_DATA,#0ffh         ; prepare port for input
    setb  LCD_RW                 ; prepare R/W for read operation

    push  acc                    ; save accumulator contents
pEw4:
    setb  LCD_E                  ; start the Enable pulse
    mov   a, LCD_DATA            ; read status nibble
    clr   LCD_E                  ; end Enable pulse
    setb  LCD_E                  ; pretend reading status low nibble
    clr   LCD_E
    jb    acc.7,pEw4             ; loop while Busy Flag is set
    pop   acc                    ; restore accumulator
    ret

; ========================================================================
; subroutine resetLCD4 - reset the LCD
; software version of the power on reset operation
;
; ========================================================================
resetLCD4:
    clr   LCD_RS                 ; LCD Register Select line
    clr   LCD_RW                 ; Read / Write line
    clr   LCD_E                  ; Enable line

    clr   LCD_DB7                ; set bit pattern for...
    clr   LCD_DB6                ; ... power-on-reset
    setb  LCD_DB5
    setb  LCD_DB4
                                 ; step 1
    setb  LCD_E                  ; start Enable pulse
    clr   LCD_E                  ; end Enable pulse
    mov   a, #4                  ; delay 4 milliseconds
    lcall mdelay
                                 ; step 2
    setb  LCD_E                  ; start Enable pulse
    clr   LCD_E                  ; end Enable pulse
    mov   a, #1                  ; delay 1 millisecond
    lcall mdelay
                                 ; step 3
    setb  LCD_E                  ; start Enable pulse
    clr   LCD_E                  ; end Enable pulse
    mov   a, #1                  ; delay 1 millisecond
    lcall mdelay

    clr   LCD_DB4                ; specify 4-bit operation
                                 ; step 4
    setb  LCD_E                  ; start Enable pulse
    clr   LCD_E                  ; end Enable pulse
    mov   a, #1                  ; delay 1 millisecond
    lcall mdelay

    mov   r0,#Config            ; FUNCTION SET
    lcall wrLCDcom4

    mov   r0,#offdsp            ; display off
    lcall wrLCDcom4

    mov   r0,#clrdsp            ; clear display, home cursor
    lcall wrLCDcom4

    mov   r0,#entryMode         ; set ENTRY MODE
    lcall wrLCDcom4

    sjmp  initlcd4

; ========================================================================
; subroutine initLCD4 - initialize the LCD
;
; ========================================================================
initLCD4:
    clr LCD_RS                   ; LCD Register Select line
    clr LCD_RW                   ; Read / Write line
    clr LCD_E                    ; Enable line

    mov   r0,#Config            ; Function Set - data bits, lines, fonts
    lcall wrLCDcom4

    mov   r0,#onDsp             ; display ON
    lcall wrLCDcom4

    mov   r0,#entryMode         ; set Entry Mode
    lcall wrLCDcom4               ; increment cursor to right,
                                 ; no display shift

    mov   r0,#clrDsp            ; clear display, home cursor
    lcall wrLCDcom4

    ret

;*****************************************
; long Delay
;*****************************************

big_delay2:
               mov   psw,#08h
               mov   r0,#0fh
x0a:
               mov   r2,#0ffh
x1a:
               mov   r5,#0ffh
x2a:
               djnz  r5,x2a
               djnz  r2,x1a
               djnz  r0,x0a
               mov   psw,#00h
               ret

;*****************************************
; short Delay
;*****************************************

Short_delay:
               mov   psw,#08h
               mov   r0,#01h
x0b:
               mov   r2,#50h
x1b:
               mov   r5,#0ffh
x2b:
               djnz  r5,x2b
               djnz  r2,x1b
               djnz  r0,x0b
               mov   psw,#00h
               ret

;******************************************
; Shorter Delay
;******************************************

mdelay:
               push  acc
               mov   a,#0c6h

mdolp:
               inc   a
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               jnz   mdolp
               nop
               pop   acc
               ret

;******************************************
;**** SUBROUTINE TO OPEN TIMEKEEPER *******
;******************************************
;
;  This subroutine executes the sequence of reads and writes which
;  is required in order to open communication with the timekeeper.
;  The subroutine returns with the timekeeper opened for data
;  access and with both the accumulator and B register modified.
;
OPEN:
                LCALL           CLOSE           ; Make sure it is closed.
                MOV             B,#4            ; Set pattern period count.
                MOV             A,#0C5H         ; Load first byte of pattern.
OPENA:
                LCALL           WBYTE           ; Send out the byte.
                XRL             A,#0FFH         ; Generate next pattern byte.
                LCALL           WBYTE           ; Send out the byte.
                SWAP            A               ; Generate next pattern byte.
                DJNZ            B,OPENA         ; Repeat until 8 bytes sent.
                RET                             ; Return.
;
;******************************************
;**** SUBROUTINE TO CLOSE TIMEKEEPER ******
;******************************************
;
;  This subroutine insures that the registers of the timekeeper
;  are closed by executing 9 successive reads of the date and time
;  registers.  The subroutine returns with both the accumulator
;  and the B register modified.
;
CLOSE:
                MOV             B,#9            ; Set up to read 9 bytes.
CLOSEA:
                LCALL           RBYTE           ; Read a byte;
                DJNZ            B,CLOSEA        ; Loop for 9 byte reads.
                RET                             ; Return.
;
;******************************************
;**** SUBROUTINE TO READ A DATA BYTE ******
;******************************************
;
;  This subroutine performs a "context switch" to the CE2 data
;  space and then reads one byte from the timekeeping device.
;  Then it switches back to the CE1 data space and returns
;  the byte read in the accumulator, with all other registers
;  unchanged.
;
RBYTE:
                PUSH            DPL             ; Save the data
                PUSH            DPH             ;    pointer on stack.
                PUSH            MCON            ; Save MCON register.
                ORL             MCON,#4         ; Switch to CE2.
                PUSH            B               ; Save the B register.
                MOV             DPL ,#4         ; Set up for data input.
                MOV             DPH, #0         ; Set high address byte.
                MOV             B,   #8         ; Set the bit count.
LI:
                PUSH            ACC             ; Save the accumulator.
                MOVX            A,   @DPTR      ; Input the data bit.
                RLC             A               ; Move it to carry.
                POP             ACC             ; Get the accumulator.
                RRC             A               ; Save the data bit.
                DJNZ            B,   LI         ; Loop for a whole byte.
                POP             B               ; Restore the B register.
                POP             MCON            ; Restore MCON register.
                POP             DPH             ; Restore the data
                POP             DPL             ;    pointer from stack.
                RET                             ; Return.
;
;******************************************
;**** SUBROUTINE TO WRITE A DATA BYTE *****
;******************************************
;
;  This subroutine performs a "context switch" to the CE2 data
;  space and then writes one byte from the accumulator to the
;  timekeeping device.  Then it switches back to the CE1 data
;  space and returns with all registers unchanged.
;
WBYTE:
                PUSH            DPL             ; Save the data
                PUSH            DPH             ;    pointer on stack.
                PUSH            MCON            ; Save MCON register.
                ORL             MCON,#4         ; Switch to CE2.
                PUSH            B               ; Save the B register.
                MOV             DPH, #0         ; Set high address byte.
                MOV             B,   #8         ; Set the bit count.
LO:
                PUSH            ACC             ; Save the accumulator.
                ANL             A,   #1         ; Set up bit for output.
                MOV             DPL, A          ; Set address to write bit.
                MOVX            A,   @DPTR      ; Output the data bit.
                POP             ACC             ; Restore the accumulator.
                RR              A               ; Position next bit.
                DJNZ            B,   LO         ; Loop for a whole byte.
                POP             B               ; Restore the B register.
                POP             MCON            ; Restore MCON register.
                POP             DPH             ; Restore the data
                POP             DPL             ;    pointer from stack.
                RET                             ; Return.


END
